//
//  DCStatusItemController.h
//  dc
//
//  Created by Work on 20/12/2010.
//  Copyright 2010 Nicholas Moore. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "NMGeometryUtils.h"
#import "NMEscapableItem.h"
#import "NSObject+NMObservePrefs.h"
#import "NMStatusBubbleWindow.h"

extern NSString *NMStatusItemClickedNotification;
extern NSString *NMStatusItemRightClickedNotification;

@protocol NMStatusItemControllerDelegate
@required
@property (readonly) NSImage *statusItemImage;
@property (readonly) NSSize statusItemImageSize;
@optional
@property (readonly) CGFloat statusItemWidth;
- (void)mouseEnteredStatusItem;
- (void)mouseExitedStatusItem;
- (void)statusItemRightClicked;
@end

@interface NMStatusItemController : NSWindowController <NSWindowDelegate, NSMenuDelegate, NMEscapableItemDelegate> {
    id <NMStatusItemControllerDelegate, NSObject> _delegate;
    
    // the status item and menu
	NSStatusItem *_statusItem;
	NSMenu *_menu;
    NMStatusBubbleWindow *_menuWindow;
    
    // image
    NSImage *_image;
    NSRect _statusImageRect;
    NSColor *_animColor;
    CGFloat _r;
    CGFloat _g;
    CGFloat _b;
    
    // menu state
	BOOL _menuIsOpen;
	BOOL _canOpenMenu;

    // icon state
    BOOL _enabled;
	BOOL _ready;
    BOOL _override;
    
    BOOL _glow;
    
    // animation
    BOOL _animating;
	NSInteger _animPos;
	NSTimer *_animTimer;
    
    BOOL _sticky;
    BOOL _temporarilyInMenu;
}
@property BOOL temporarilyInMenu;
@property BOOL enabled;
@property BOOL ready;
@property BOOL override;
@property BOOL glow;
@property BOOL sticky;
@property (unsafe_unretained, readonly) NSWindow *itemWindow;
@property BOOL iconVisible;

+ (NMStatusItemController *)sharedInstance;

- (void)attachMenu:(NSMenu *)menu;
- (void)attachWindow:(NMStatusBubbleWindow *)window;
- (void)closeAttachedWindow;
- (void)openAttachedWindow;
- (void)showAttachedMenu:(BOOL)force;
- (void)showAttachedMenu;
- (void)doAlternateAction;
- (void)doRightClickAction;
- (void)startGlowAnimationWithColor:(NSColor *)color;
- (void)stopAnimation;
- (NSRect)statusItemRect;
- (void)drawInRect:(NSRect)rect;

@end
